# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.chinese_clip.modeling import (
    ChineseClipForPretrain,
    ChineseClipForClassification,
    ChineseClipForTextClassification,
    ChineseClipForImageClassification,
)
from unitorch.models.chinese_clip.processing import ChineseClipProcessor
