# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.mask2former.modeling import Mask2FormerForSegmentation
from unitorch.models.mask2former.processing import Mask2FormerProcessor
